/*
 * Exchange Web Services Managed API
 *
 * Copyright (c) Microsoft Corporation
 * All rights reserved.
 *
 * MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */




    /// <summary>
    /// Represents the set of actions available for a rule.
    /// </summary>
 class RuleActions extends ComplexProperty
    {
        /// <summary>
        /// SMS recipient address type.
        /// </summary>
        /* private */ const String MobileType = "MOBILE";

        /// <summary>
        /// The AssignCategories action.
        /// </summary>
        /* private */ StringList assignCategories;

        /// <summary>
        /// The CopyToFolder action.
        /// </summary>
        /* private */ FolderId copyToFolder;

        /// <summary>
        /// The Delete action.
        /// </summary>
        /* private */ bool delete;

        /// <summary>
        /// The ForwardAsAttachmentToRecipients action.
        /// </summary>
        /* private */ EmailAddressCollection forwardAsAttachmentToRecipients;

        /// <summary>
        /// The ForwardToRecipients action.
        /// </summary>
        /* private */ EmailAddressCollection forwardToRecipients;

        /// <summary>
        /// The MarkImportance action.
        /// </summary>
        /* private */ Importance? markImportance;

        /// <summary>
        /// The MarkAsRead action.
        /// </summary>
        /* private */ bool markAsRead;

        /// <summary>
        /// The MoveToFolder action.
        /// </summary>
        /* private */ FolderId moveToFolder;

        /// <summary>
        /// The PermanentDelete action.
        /// </summary>
        /* private */ bool permanentDelete;

        /// <summary>
        /// The RedirectToRecipients action.
        /// </summary>
        /* private */ EmailAddressCollection redirectToRecipients;

        /// <summary>
        /// The SendSMSAlertToRecipients action.
        /// </summary>
        /* private */ Collection<MobilePhone> sendSMSAlertToRecipients;

        /// <summary>
        /// The ServerReplyWithMessage action.
        /// </summary>
        /* private */ ItemId serverReplyWithMessage;

        /// <summary>
        /// The StopProcessingRules action.
        /// </summary>
        /* private */ bool stopProcessingRules;

        /// <summary>
        /// Initializes a new instance of the <see cref="RulePredicates"/> class.
        /// </summary>
        RuleActions()
            : super()
        {
            this.assignCategories = new StringList();
            this.forwardAsAttachmentToRecipients = new EmailAddressCollection(XmlElementNames.Address);
            this.forwardToRecipients = new EmailAddressCollection(XmlElementNames.Address);
            this.redirectToRecipients = new EmailAddressCollection(XmlElementNames.Address);
            this.sendSMSAlertToRecipients = new Collection<MobilePhone>();
        }

        /// <summary>
        /// Gets the categories that should be stamped on incoming messages.
        /// To disable stamping incoming messages with categories, set
        /// AssignCategories to null.
        /// </summary>
 StringList AssignCategories
        {
            get
            {
                return this.assignCategories;
            }
        }

        /// <summary>
        /// Gets or sets the Id of the folder incoming messages should be copied to.
        /// To disable copying incoming messages to a folder, set CopyToFolder to null.
        /// </summary>
 FolderId CopyToFolder
        {
            get
            {
                return this.copyToFolder;
            }

            set
            {
                this.SetFieldValue<FolderId>(ref this.copyToFolder, value);
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether incoming messages should be
        /// automatically moved to the Deleted Items folder.
        /// </summary>
 bool Delete
        {
            get
            {
                return this.delete;
            }

            set
            {
                this.SetFieldValue<bool>(ref this.delete, value);
            }
        }

        /// <summary>
        /// Gets the e-mail addresses to which incoming messages should be
        /// forwarded as attachments. To disable forwarding incoming messages
        /// as attachments, empty the ForwardAsAttachmentToRecipients list.
        /// </summary>
 EmailAddressCollection ForwardAsAttachmentToRecipients
        {
            get
            {
                return this.forwardAsAttachmentToRecipients;
            }
        }

        /// <summary>
        /// Gets the e-mail addresses to which incoming messages should be forwarded.
        /// To disable forwarding incoming messages, empty the ForwardToRecipients list.
        /// </summary>
 EmailAddressCollection ForwardToRecipients
        {
            get
            {
                return this.forwardToRecipients;
            }
        }

        /// <summary>
        /// Gets or sets the importance that should be stamped on incoming
        /// messages. To disable the stamping of incoming messages with an
        /// importance, set MarkImportance to null.
        /// </summary>
 Importance? MarkImportance
        {
            get
            {
                return this.markImportance;
            }

            set
            {
                this.SetFieldValue<Importance?>(ref this.markImportance, value);
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether incoming messages should be
        /// marked as read.
        /// </summary>
 bool MarkAsRead
        {
            get
            {
                return this.markAsRead;
            }

            set
            {
                this.SetFieldValue<bool>(ref this.markAsRead, value);
            }
        }

        /// <summary>
        /// Gets or sets the Id of the folder to which incoming messages should be
        /// moved. To disable the moving of incoming messages to a folder, set
        /// CopyToFolder to null.
        /// </summary>
 FolderId MoveToFolder
        {
            get
            {
                return this.moveToFolder;
            }

            set
            {
                this.SetFieldValue<FolderId>(ref this.moveToFolder, value);
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether incoming messages should be
        /// permanently deleted. When a message is permanently deleted, it is never
        /// saved into the recipient's mailbox. To delete a message after it has
        /// been saved into the recipient's mailbox, use the Delete action.
        /// </summary>
 bool PermanentDelete
        {
            get
            {
                return this.permanentDelete;
            }

            set
            {
                this.SetFieldValue<bool>(ref this.permanentDelete, value);
            }
        }

        /// <summary>
        /// Gets the e-mail addresses to which incoming messages should be
        /// redirecteded. To disable redirection of incoming messages, empty
        /// the RedirectToRecipients list. Unlike forwarded mail, redirected mail
        /// maintains the original sender and recipients.
        /// </summary>
 EmailAddressCollection RedirectToRecipients
        {
            get
            {
                return this.redirectToRecipients;
            }
        }

        /// <summary>
        /// Gets the phone numbers to which an SMS alert should be sent. To disable
        /// sending SMS alerts for incoming messages, empty the
        /// SendSMSAlertToRecipients list.
        /// </summary>
 Collection<MobilePhone> SendSMSAlertToRecipients
        {
            get
            {
                return this.sendSMSAlertToRecipients;
            }
        }

        /// <summary>
        /// Gets or sets the Id of the template message that should be sent
        /// as a reply to incoming messages. To disable automatic replies, set
        /// ServerReplyWithMessage to null.
        /// </summary>
 ItemId ServerReplyWithMessage
        {
            get
            {
                return this.serverReplyWithMessage;
            }

            set
            {
                this.SetFieldValue<ItemId>(ref this.serverReplyWithMessage, value);
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether subsequent rules should be
        /// evaluated.
        /// </summary>
 bool StopProcessingRules
        {
            get
            {
                return this.stopProcessingRules;
            }

            set
            {
                this.SetFieldValue<bool>(ref this.stopProcessingRules, value);
            }
        }

        /// <summary>
        /// Tries to read element from XML.
        /// </summary>
        /// <param name="reader">The reader.</param>
        /// <returns>True if element was read.</returns>
@override
        bool TryReadElementFromXml(EwsServiceXmlReader reader)
        {
            switch (reader.LocalName)
            {
                case XmlElementNames.AssignCategories:
                    this.assignCategories.LoadFromXml(reader, reader.LocalName);
                    return true;
                case XmlElementNames.CopyToFolder:
                    reader.ReadStartElement(XmlNamespace.NotSpecified, XmlElementNames.FolderId);
                    this.copyToFolder = new FolderId();
                    this.copyToFolder.LoadFromXml(reader, XmlElementNames.FolderId);
                    reader.ReadEndElement(XmlNamespace.NotSpecified, XmlElementNames.CopyToFolder);
                    return true;
                case XmlElementNames.Delete:
                    this.delete = reader.ReadElementValue<bool>();
                    return true;
                case XmlElementNames.ForwardAsAttachmentToRecipients:
                    this.forwardAsAttachmentToRecipients.LoadFromXml(reader, reader.LocalName);
                    return true;
                case XmlElementNames.ForwardToRecipients:
                    this.forwardToRecipients.LoadFromXml(reader, reader.LocalName);
                    return true;
                case XmlElementNames.MarkImportance:
                    this.markImportance = reader.ReadElementValue<Importance>();
                    return true;
                case XmlElementNames.MarkAsRead:
                    this.markAsRead = reader.ReadElementValue<bool>();
                    return true;
                case XmlElementNames.MoveToFolder:
                    reader.ReadStartElement(XmlNamespace.NotSpecified, XmlElementNames.FolderId);
                    this.moveToFolder = new FolderId();
                    this.moveToFolder.LoadFromXml(reader, XmlElementNames.FolderId);
                    reader.ReadEndElement(XmlNamespace.NotSpecified, XmlElementNames.MoveToFolder);
                    return true;
                case XmlElementNames.PermanentDelete:
                    this.permanentDelete = reader.ReadElementValue<bool>();
                    return true;
                case XmlElementNames.RedirectToRecipients:
                    this.redirectToRecipients.LoadFromXml(reader, reader.LocalName);
                    return true;
                case XmlElementNames.SendSMSAlertToRecipients:
                    EmailAddressCollection smsRecipientCollection = new EmailAddressCollection(XmlElementNames.Address);
                    smsRecipientCollection.LoadFromXml(reader, reader.LocalName);
                    this.sendSMSAlertToRecipients = ConvertSMSRecipientsFromEmailAddressCollectionToMobilePhoneCollection(smsRecipientCollection);
                    return true;
                case XmlElementNames.ServerReplyWithMessage:
                    this.serverReplyWithMessage = new ItemId();
                    this.serverReplyWithMessage.LoadFromXml(reader, reader.LocalName);
                    return true;
                case XmlElementNames.StopProcessingRules:
                    this.stopProcessingRules = reader.ReadElementValue<bool>();
                    return true;
                default:
                    return false;
            }
        }

        /// <summary>
        /// Writes elements to XML.
        /// </summary>
        /// <param name="writer">The writer.</param>
@override
        void WriteElementsToXml(EwsServiceXmlWriter writer)
        {
            if (this.AssignCategories.Count > 0)
            {
                this.AssignCategories.WriteToXml(writer, XmlElementNames.AssignCategories);
            }

            if (this.CopyToFolder != null)
            {
                writer.WriteStartElement(XmlNamespace.Types, XmlElementNames.CopyToFolder);
                this.CopyToFolder.WriteToXml(writer);
                writer.WriteEndElement();
            }

            if (this.Delete != false)
            {
                writer.WriteElementValue(
                    XmlNamespace.Types,
                    XmlElementNames.Delete,
                    this.Delete);
            }

            if (this.ForwardAsAttachmentToRecipients.Count > 0)
            {
                this.ForwardAsAttachmentToRecipients.WriteToXml(writer, XmlElementNames.ForwardAsAttachmentToRecipients);
            }

            if (this.ForwardToRecipients.Count > 0)
            {
                this.ForwardToRecipients.WriteToXml(writer, XmlElementNames.ForwardToRecipients);
            }

            if (this.MarkImportance.HasValue)
            {
                writer.WriteElementValue(
                    XmlNamespace.Types,
                    XmlElementNames.MarkImportance,
                    this.MarkImportance.Value);
            }

            if (this.MarkAsRead != false)
            {
                writer.WriteElementValue(
                    XmlNamespace.Types,
                    XmlElementNames.MarkAsRead,
                    this.MarkAsRead);
            }

            if (this.MoveToFolder != null)
            {
                writer.WriteStartElement(XmlNamespace.Types, XmlElementNames.MoveToFolder);
                this.MoveToFolder.WriteToXml(writer);
                writer.WriteEndElement();
            }

            if (this.PermanentDelete != false)
            {
                writer.WriteElementValue(
                    XmlNamespace.Types,
                    XmlElementNames.PermanentDelete,
                    this.PermanentDelete);
            }

            if (this.RedirectToRecipients.Count > 0)
            {
                this.RedirectToRecipients.WriteToXml(writer, XmlElementNames.RedirectToRecipients);
            }

            if (this.SendSMSAlertToRecipients.Count > 0)
            {
                EmailAddressCollection emailCollection = ConvertSMSRecipientsFromMobilePhoneCollectionToEmailAddressCollection(this.SendSMSAlertToRecipients);
                emailCollection.WriteToXml(writer, XmlElementNames.SendSMSAlertToRecipients);
            }

            if (this.ServerReplyWithMessage != null)
            {
                this.ServerReplyWithMessage.WriteToXml(writer, XmlElementNames.ServerReplyWithMessage);
            }

            if (this.StopProcessingRules != false)
            {
                writer.WriteElementValue(
                    XmlNamespace.Types,
                    XmlElementNames.StopProcessingRules,
                    this.StopProcessingRules);
            }
        }

        /// <summary>
        /// Validates this instance.
        /// </summary>
@override
        void InternalValidate()
        {
            super.InternalValidate();
            EwsUtilities.ValidateParam(this.forwardAsAttachmentToRecipients, "ForwardAsAttachmentToRecipients");
            EwsUtilities.ValidateParam(this.forwardToRecipients, "ForwardToRecipients");
            EwsUtilities.ValidateParam(this.redirectToRecipients, "RedirectToRecipients");
            for (MobilePhone sendSMSAlertToRecipient in this.sendSMSAlertToRecipients)
            {
                EwsUtilities.ValidateParam(sendSMSAlertToRecipient, "SendSMSAlertToRecipient");
            }
        }

        /// <summary>
        /// Convert the SMS recipient list from EmailAddressCollection type to MobilePhone collection type.
        /// </summary>
        /// <param name="emailCollection">Recipient list in EmailAddressCollection type.</param>
        /// <returns>A MobilePhone collection object containing all SMS recipient in MobilePhone type. </returns>
        /* private */ static Collection<MobilePhone> ConvertSMSRecipientsFromEmailAddressCollectionToMobilePhoneCollection(EmailAddressCollection emailCollection)
        {
            Collection<MobilePhone> mobilePhoneCollection = new Collection<MobilePhone>();
            for (EmailAddress emailAddress in emailCollection)
            {
                mobilePhoneCollection.Add(new MobilePhone(emailAddress.Name, emailAddress.Address));
            }

            return mobilePhoneCollection;
        }

        /// <summary>
        /// Convert the SMS recipient list from MobilePhone collection type to EmailAddressCollection type.
        /// </summary>
        /// <param name="recipientCollection">Recipient list in a MobilePhone collection type.</param>
        /// <returns>An EmailAddressCollection object containing recipients with "MOBILE" address type. </returns>
        /* private */ static EmailAddressCollection ConvertSMSRecipientsFromMobilePhoneCollectionToEmailAddressCollection(Collection<MobilePhone> recipientCollection)
        {
            EmailAddressCollection emailCollection = new EmailAddressCollection(XmlElementNames.Address);
            for (MobilePhone recipient in recipientCollection)
            {
                EmailAddress emailAddress = new EmailAddress(
                    recipient.Name,
                    recipient.PhoneNumber,
                    RuleActions.MobileType);
                emailCollection.Add(emailAddress);
            }

            return emailCollection;
        }
    }
